from kpl_helper.base import _MsgType, _send_metric, get_config


# metric plot type
PLOT_LINE = 'line'


class MetricFigure:
    def __init__(self, title, y1_name, y1_type=PLOT_LINE, y2_name=None, y2_type=None):
        if not get_config().get_inner():
            return
        """
        :param title:      图表的名称(最多两个纵轴)
        :param y1_name:    metric纵轴1名称, 如 Accuracy
        :param y1_type:    metric纵轴1类型, 如 PLOT_LINE为折线图
        :param y2_name:    metric纵轴2名称, 如 Softmax_Loss
        :param y2_type:    metric纵轴2类型, 如 PLOT_LINE为折线图
        """
        self.y1_name = y1_name
        self.y2_name = y2_name
        self.title = title
        if not isinstance(title, str) and len(title) == 0:
            raise ValueError("title must be string")
        if not isinstance(y1_name, str) and len(y1_name) == 0:
            raise ValueError("y1 must be string")
        if y1_type not in (PLOT_LINE,):
            raise KeyError("y1_type not exists")
        series = [{
            "name": y1_name,
            "type": y1_type,
        }]
        if y2_name:
            if not isinstance(y2_name, str) and len(y2_name) == 0:
                raise ValueError("y2 must be string")
            if y2_type not in (PLOT_LINE,):
                raise KeyError("y2_type not exists")
            series.append({
                "name": y2_name,
                "type": y2_type,
            })
        body = {
            "msgType": _MsgType.NewMetric,
            "title": title,
            "series": series
        }
        _send_metric(body)

    def push_metric(self, x, y1, y2=None):
        if not get_config().get_inner():
            return
        """
        :param x:   横坐标，一般为迭代次数
        :param y1:  x对应的y1的值
        :param y2:  x对应的y2的值
        :return:
        """
        items = [{
            "msgType": _MsgType.MetricData,
            "title": self.title,
            "seriesName": self.y1_name,
            "value": [x, y1]
        }]
        if y2 and self.y2_name:
            items.append({
                "msgType": _MsgType.MetricData,
                "title": self.title,
                "seriesName": self.y2_name,
                "value": [x, y2]
            })
        _send_metric(items)


if __name__ == '__main__':
    figure = MetricFigure("LOSS&Accuracy", "LOSS", PLOT_LINE, "Accuracy", PLOT_LINE)
    figure2 = MetricFigure("LOSS图表", "LOSS", PLOT_LINE)
    import time
    import random
    for i in range(100000):
        print("send metric")
        figure.push_metric(i, random.randint(0, i + 5), random.random() + i * 0.1)
        figure2.push_metric(i, random.randint(0, 10))
        time.sleep(3)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
from kpl_helper.base import get_config
import sys
import json
import os
import logging
import yaml
logger = logging.getLogger("kpl-helper")


class DotDict(dict):
    __getattr__ = dict.__getitem__
    __setattr__ = dict.__setitem__
    __delattr__ = dict.__delitem__

    def __init__(self, dct):
        super(DotDict, self).__init__()
        for key, value in dct.items():
            if hasattr(value, 'keys'):
                value = DotDict(value)
            self[key] = value

    def __delitem__(self, key):
        super(DotDict, self).__delitem__(key)
        del self.__dict__[key]

    # setstate and getstate is for pickle
    def __getstate__(self):
        pass

    def __setstate__(self, *args, **kwargs):
        pass


def _load_json(str):
    if sys.version_info.major >= 3:
        return json.loads(str)

    def json_loads_byteified(json_text):
        return _byteify(
            json.loads(json_text, object_hook=_byteify),
            ignore_dicts=True
        )

    def _byteify(data, ignore_dicts=False):
        if isinstance(data, unicode):
            return data.encode('utf-8')
        if isinstance(data, list):
            return [_byteify(item, ignore_dicts=True) for item in data]
        if isinstance(data, dict) and not ignore_dicts:
            return {
                _byteify(key, ignore_dicts=True): _byteify(value, ignore_dicts=True)
                for key, value in data.iteritems()
            }
        return data

    return json_loads_byteified(str)


_parameter = None


def get_parameter(default=None):
    global _parameter
    if _parameter is not None:
        return _parameter
    if get_config().get_inner():
        parameter = get_config().get_parameter()
        _parameter = DotDict(_load_json(parameter)) if parameter else None
        return _parameter
    if default:
        with open(default) as fi:
            param = yaml.safe_load(fi)
            _parameter = DotDict(param)
            return _parameter
    raise Exception("Not found parameter")


def write_parameter():
    argv = sys.argv
    if argv[1] == 'write' and len(argv) > 2:
        output_yml = argv[2]
    else:
        logger.error('Write Failed !!!\nUsage `helper write <yml file>` to write parameters to yml')
        sys.exit(1)
    parameter = get_config().get_parameter()
    try:
        if parameter is None:
            raise ValueError('Parameter is None')
        path = os.path.dirname(output_yml)
        if path and not os.path.isdir(path):
            os.makedirs(path)
        with open(output_yml, 'w') as fo:
            fo.write(parameter)
    except Exception as e:
        logger.error('Could get parameters. {}'.format(e))




from kpl_helper import metric
figure = metric.MetricFigure("title", y1_name="loss")
iter_num = 100
for no in range(1, iter_num):
    ce_loss = 10.0 / no
    figure.push_metric(x=no, y1=ce_loss)


from kpl_helper.progress import send_progress
iter_num = 100
for no in range(1, iter_num):
    send_progress(float(no) / iter_num)



from kpl_helper.base import _send_result, _ResultType, get_config
from kpl_helper.base import logger


def send_scalar_result(**results):
    """
    发送单数值测试结果
    eg: send_scalar_result(top1=0.92, top2=0.98)
    """
    if not get_config().get_inner():
        return
    for k, v in results.items():
        _send_result(_ResultType.SCALAR_RESULT, k, v)


def send_curve_result(title, x_name, x_points, y_name, y_points):
    """
    发送测试协议输出为曲线的测试结果
    eg: send_curve_result("Recall and Precision", "recall", [0.0, 0.2, 0.4, 0.6, 0.8, 1.0], "precision",[0.0, 0.8, 0.89, 0.94, 0.96, 1.0])
    """
    if not get_config().get_inner():
        return
    if len(x_points) != len(y_points):
        logger.error("send curve result len(x_points)={} VS len(y_points)={}".format(len(x_points), len(y_points)))
        return
    data = []
    for x, y in zip(x_points, y_points):
        data.append([x, y])
    value = {
        "x": x_name,
        "y": y_name,
        "data": data
    }
    _send_result(_ResultType.CURVE_RESULT, title, value)


if __name__ == '__main__':
    send_scalar_result(top1=0.92, top2=0.98)
    send_curve_result("Recall and Precision",
                      "recall", [0.0, 0.2, 0.4, 0.6, 0.8, 1.0],
                      "precision", [0.0, 0.8, 0.89, 0.94, 0.96, 1.0])



from kpl_helper.base import get_config


class RectangleBoxObjectDetectionResponse:
    _response_type = "rectangle_box_detection"

    def __init__(self, height, width, depth):
        self.height = height
        self.width = width
        self.depth = depth
        self.objects = []

    def add_object(self, name, xmin, ymin, xmax, ymax, score):
        self.objects.append({
            "name": name,
            "score": float(score),
            "xmin": int(xmin),
            "ymin": int(ymin),
            "xmax": int(xmax),
            "ymax": int(ymax)
        })

    def dumps(self):
        return {
            "type": type(self)._response_type,
            "height": self.height,
            "width": self.width,
            "depth": self.depth,
            "objects": self.objects
        }


class ClassificationResponse:
    _response_type = "classification"

    @staticmethod
    def dumps(height, width, depth, class_id, class_name=None):
        return {
            "type": ClassificationResponse._response_type,
            "height": height,
            "width": width,
            "depth": depth,
            "class_id":class_id,
            "class_name": class_name,
        }


class SegmentationResponse:
    _response_type = "segmentation"

    @staticmethod
    def dumps(height, width, depth, mask):
        return {
            "type": SegmentationResponse._response_type,
            "height": height,
            "width": width,
            "depth": depth,
            "mask": mask,
        }




from kpl_helper.base import get_config
import json
import os
import logging
logger = logging.getLogger("kpl-helper")


def get_input_path(key, default=""):
    inner = get_config().get_inner()
    if not inner:
        return default
    in_keys = get_config().get_input_keys()
    index = in_keys.index(key)
    if index < 0:
        raise Exception("get_input_dir(). key not exists")
    root = get_config().get_input_root()
    path = os.path.join(root, str(index))
    if not os.path.exists(path):
        raise Exception("input direction not exists: {}".format(path))
    return path


def get_output_path(key, default=""):
    inner = get_config().get_inner()
    if not inner:
        return default
    in_keys = get_config().get_output_keys()
    index = in_keys.index(key)
    if index < 0:
        raise Exception("get_input_dir(). key not exists")
    root = get_config().get_output_root()
    path = os.path.join(root, str(index))
    if not os.path.exists(path):
        os.makedirs(path)
    return path


# def send_data_attribute(output_index=1, class_num=None, class_name_dict=None, sample_num=None):
#     """
#     将数据的属性信息发送给服务器
#     :param output_index:    第index个输出，如划分训练集和测试集会有多个输出. 从1开始
#     :param class_num:   类别数量.
#     :param class_name_dict:  id与类名对应字典. 如: {0: "airplane", 1: "sofa", ...}
#     :param sample_num:  数据集包含样本总数. 注意: 样本总数并非文件总数，如检测数据集，样本总数=图片文件总数=XML标注文件总数
#     :return:
#     """
#     if not KPL_INNER:
#         return
#     dataset_api = get_dataset_api()
#     for k, v in {"class_name": class_num, "class_name_dict": class_name_dict, "sample_num": sample_num}.items():
#         try:
#             resp = session.post('{}/updateDatasetAttribute'.format(dataset_api),
#                                 json={
#                                     "token": get_dataset_token(),
#                                     "output": str(output_index),
#                                     "name": k,
#                                     "value": json.dumps(v)
#                                 },
#                                 timeout=5)
#             logger.info("send data attribute, {} = {}".format(k, v))
#             if resp.status_code != 200:
#                 logger.error("send data attribute http code: {}. content: {}".format(resp.status_code, resp.content))
#         except requests.RequestException as e:
#             logger.error('Could not reach dataset api. detail: {}'.format(e))



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(name='kpl-helper-test',
      version='0.0.5',
      platforms='any',
      packages=find_packages(),
      install_requires=[
          'requests',
          'flask',
          'flask-cors',
          'PyYAML',
          'kpl-dataset'
      ],
      entry_points={
          "console_scripts": [
              "helper = kpl_helper.parameter:write_parameter",
          ],
      },
      classifiers=[
          'Programming Language :: Python',
          'Operating System :: OS Independent',
      ])



from kpl_helper.base import _ResultType, _send_result, logger, get_config


def send_progress(progress):
    """
    :param progress: 百分比进度，值范围：0.0 ~ 1.0
    :return:
    """
    if not get_config().get_inner():
        return
    if not (0.0 <= progress <= 1.0):
        logger.error("progress value error. should in range [0.0: 1.0]. but get {}".format(progress))
        if progress < 0.0: progress = 0.0
        if progress > 1.0: progress = 1.0
    _send_result(_ResultType.PROGRESS, "progress", progress)


if __name__ == '__main__':
    for i in range(100):
        print("send progress")
        send_progress(i * 0.01)



import requests
from requests.adapters import HTTPAdapter
import logging
import sys
import os
import json

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger("kpl-helper")

session = requests.Session()
session.mount('http://', HTTPAdapter(max_retries=3))
session.mount('https://', HTTPAdapter(max_retries=3))


class _BaseConfig:
    def __init__(self):
        self._inner = os.getenv("KPL_INNER")
        self._input_root = os.getenv("KPL_INPUT_ROOT")
        input_keys = os.getenv("KPL_INPUT_KEYS")
        self._input_keys = json.loads(input_keys) if input_keys else []
        self._output_root = os.getenv("KPL_OUTPUT_ROOT", "/")
        output_keys = os.getenv("KPL_OUTPUT_KEYS")
        self._output_keys = json.loads(output_keys) if output_keys else []
        self._parameter = os.getenv("KPL_PARAMETER")
        self._metric_api = os.getenv("KPL_METRIC_API")
        self._metric_token = os.getenv("KPL_METRIC_TOKEN")

    def get_inner(self):
        return False if self._inner is None else True

    def get_input_root(self):
        return self._input_root

    def get_input_num(self):
        return len(self._input_keys)

    def get_input_keys(self):
        return self._input_keys

    def get_output_root(self):
        return self._output_root

    def get_output_num(self):
        return len(self._output_keys)

    def get_output_keys(self):
        return self._output_keys

    def get_parameter(self):
        return self._parameter

    def get_metric_api(self):
        return self._metric_api

    def get_metric_token(self):
        return self._metric_token

    def get_cluster(self):
        pass


__base_config = None


def get_config():
    if __base_config is None:
        raise Exception("Please call ready() function before using helper package.")
    return __base_config


def ready():
    print("kpl helper is ready for using.")
    global __base_config
    __base_config = _BaseConfig()


def done():
    print("execute kpl done()")
    config = get_config()
    output_num = config.get_output_num()
    if output_num == 0:
        return
    summary_dir = os.path.join(config.get_output_root(), "summary")
    if not os.path.exists(summary_dir):
        os.makedirs(summary_dir)
    output_keys = config.get_output_keys()
    with open("{}/outputs.json".format(summary_dir), "w") as fo:
        json.dump({
            "output": [{
                "name": output_keys[i],
                "uuid": "{}".format(i)
            } for i in range(output_num)],
        }, fo)


class _MsgType:
    NewMetric = "NewMetric"
    MetricData = "MetricData"


class _ResultType:
    SCALAR_RESULT = 'scalar_result'  # 用于如Rank1，Rank5，LFW，MegaFace测试协议输出的单精度值测试结果
    CURVE_RESULT = 'curve_result'  # 用于如ROC测试协议输出的测试曲线
    PROGRESS = 'progress'


def _send_metric(body):
    if not get_config().get_inner():
        return
    if len(get_config().get_metric_api()) == 0 or len(get_config().get_metric_token()) == 0:
        raise Exception("You should run your algorithm inner SeeTaaS or AutoDL platform")
    try:
        if not isinstance(body, list):
            body = [body]
        resp = session.post('{}/uploadTaskMetrics'.format(get_config().get_metric_api()),
                            json={
                                "token": get_config().get_metric_token(),
                                "items": body
                            },
                            timeout=5)
        if resp.status_code != 200:
            logger.error("send metrics http code: {}. content: {}".format(resp.status_code, resp.content))
    except requests.RequestException as e:
        logger.error('Could not reach metric api. detail: {}'.format(e))


def _send_result(result_type, name, value):
    if not get_config().get_inner():
        return
    if len(get_config().get_metric_api()) == 0 or len(get_config().get_metric_token()) == 0:
        raise Exception("You should run your algorithm inner SeeTaaS or AutoDL platform")
    try:
        resp = session.post('{}/updateTaskAttribute'.format(get_config().get_metric_api()),
                            json={
                                "token": get_config().get_metric_token(),
                                "type": result_type,
                                "name": name,
                                "value": value
                            },
                            timeout=5)
        if resp.status_code != 200:
            logger.error("send evaluate result http code: {}. content: {}".format(resp.status_code, resp.content))
    except requests.RequestException as e:
        logger.error('Could not reach evaluate result api. detail: {}'.format(e))
